@@ -92,29 +92,29 @@ void DirReadJob::deletingChild( FileInfo *deletedChild )
 bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
 {
     if ( parent->device() == child->device() )
-        return false;
+	return false;
 
-    QString childDevice  = device( child );
+    QString childDevice	 = device( child );
     QString parentDevice = device( parent->findNearestMountPoint() );;
 
     if ( parentDevice.isEmpty() )
-        parentDevice = _tree->device();
+	parentDevice = _tree->device();
 
     bool crossing = true;
 
     if ( ! parentDevice.isEmpty() && ! childDevice.isEmpty() )
-        crossing = parentDevice != childDevice;
+	crossing = parentDevice != childDevice;
 
     if ( crossing )
     {
-            logInfo() << "File system boundary at mount point " << child
-                      << " on device " << ( childDevice.isEmpty() ? "<unknown>" : childDevice )
-                      << endl;
+	    logInfo() << "File system boundary at mount point " << child
+		      << " on device " << ( childDevice.isEmpty() ? "<unknown>" : childDevice )
+		      << endl;
     }
     else
     {
-        logInfo() << "Mount point " << child
-                  << " is still on the same device " << childDevice << endl;
+	logInfo() << "Mount point " << child
+		  << " is still on the same device " << childDevice << endl;
     }
 
     return crossing;
@@ -127,10 +127,10 @@ QString DirReadJob::device( const DirInfo * dir ) const
 
     if ( dir )
     {
-        const MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
+	const MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
 
-        if ( mountPoint )
-            device = mountPoint->device();
+	if ( mountPoint )
+	    device = mountPoint->device();
     }
 
     return device;
@@ -166,24 +166,24 @@ void LocalDirReadJob::startReading()
 
     if ( access( dirName.toUtf8(), X_OK | R_OK ) != 0 )
     {
-        ok = false;
-        logWarning() << "No permission to read directory " << dirName << endl;
+	ok = false;
+	logWarning() << "No permission to read directory " << dirName << endl;
 	_dir->setReadState( DirError );
 	finishReading( _dir );
     }
 
     if ( ok )
     {
-        _diskDir = opendir( dirName.toUtf8() );
+	_diskDir = opendir( dirName.toUtf8() );
 
-        if ( ! _diskDir )
-        {
-            ok = false;
-            _dir->setReadState( DirError );
-            logWarning() << "opendir(" << dirName << ") failed" << endl;
-            // opendir() doesn't set 'errno' according to POSIX  :-(
-            finishReading( _dir );
-        }
+	if ( ! _diskDir )
+	{
+	    ok = false;
+	    _dir->setReadState( DirError );
+	    logWarning() << "opendir(" << dirName << ") failed" << endl;
+	    // opendir() doesn't set 'errno' according to POSIX	 :-(
+	    finishReading( _dir );
+	}
     }
 
     if ( ok )
@@ -197,7 +197,8 @@ void LocalDirReadJob::startReading()
 	    if ( entryName != "."  &&
 		 entryName != ".."   )
 	    {
-		QString fullName = dirName + "/" + entryName;
+		QString fullName = dirName == "/" ? "" : dirName; // Avoid leading // when in root dir
+		fullName += "/" + entryName;
 
 		if ( lstat( fullName.toUtf8(), &statInfo ) == 0 )	      // lstat() OK
 		{
@@ -259,24 +260,34 @@ void LocalDirReadJob::startReading()
 			    {
 				logDebug() << "Using cache file " << fullName << " for " << dirName << endl;
 
-				cacheReadJob->reader()->rewind();  // Read offset was moved by firstDir()
-				_tree->addJob( cacheReadJob );	   // Job queue will assume ownership of cacheReadJob
-
-				if ( _dir->parent() )
-				    _dir->parent()->setReadState( DirReading );
-
-				//
-				// Clean up partially read directory content
-				//
-
 				DirTree * tree = _tree;	 // Copy data members to local variables:
-				DirInfo * dir  = _dir;	 // This object will be deleted soon by killAll()
+				DirInfo * dir  = _dir;	 // This object might be deleted soon by killAll()
 
-				_queue->killAll( _dir, cacheReadJob );	// Will delete this job as well!
-				// All data members of this object are invalid from here on!
+                                if ( _tree->isTopLevel( _dir ) )
+                                {
+                                    logDebug() << "Clearing complete tree" << endl;
 
-				logDebug() << "Deleting subtree " << dir << endl;
-				tree->deleteSubtree( dir );
+                                    _tree->clear();
+                                    _tree->readCache( fullName );
+                                }
+                                else
+                                {
+                                    cacheReadJob->reader()->rewind();  // Read offset was moved by firstDir()
+                                    _tree->addJob( cacheReadJob );	   // Job queue will assume ownership of cacheReadJob
+
+                                    if ( _dir->parent() )
+                                        _dir->parent()->setReadState( DirReading );
+
+                                    //
+                                    // Clean up partially read directory content
+                                    //
+
+                                    _queue->killAll( _dir, cacheReadJob );	// Will delete this job as well!
+                                    // All data members of this object are invalid from here on!
+
+                                    logDebug() << "Deleting subtree " << dir << endl;
+                                    tree->deleteSubtree( dir );
+                                }
 
 				return;
 			    }
@@ -381,7 +392,7 @@ FileInfo * LocalDirReadJob::stat( const QString & url,
     }
     else // lstat() failed
     {
-        THROW( SysCallFailedException( "lstat", url ) );
+	THROW( SysCallFailedException( "lstat", url ) );
 	return 0; // NOTREACHED
     }
 }
@@ -534,6 +545,7 @@ void DirReadJobQueue::killAll( DirInfo * subtree, DirReadJob * exceptJob )
 	return;
 
     QMutableListIterator<DirReadJob *> it( _queue );
+    int count = 0;
 
     while ( it.hasNext() )
     {
@@ -541,17 +553,20 @@ void DirReadJobQueue::killAll( DirInfo * subtree, DirReadJob * exceptJob )
 
 	if ( exceptJob && job == exceptJob )
 	{
-	    logDebug() << "NOT killing read job " << job->dir() << endl;
+	    logDebug() << "NOT killing " << job << endl;
 	    continue;
 	}
 
 	if ( job->dir() && job->dir()->isInSubtree( subtree ) )
 	{
-	    // logDebug() << "Killing read job " << job->dir() << endl;
+	    // logDebug() << "Killing " << job << endl;
+	    ++count;
 	    it.remove();
 	    delete job;
 	}
     }
+
+    logDebug() << "Killed " << count << " read jobs for " << subtree << endl;
 }
 
 
@@ -579,3 +594,12 @@ void DirReadJobQueue::jobFinishedNotify( DirReadJob *job )
     }
 }
 
+
+void DirReadJobQueue::deletingChildNotify( FileInfo * child )
+{
+    if ( child && child->isDirInfo() )
+    {
+	logDebug() << "Killing all pending read jobs for " << child << endl;
+	killAll( child->toDirInfo() );
+    }
+}
