@@ -9,6 +9,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <pwd.h>	// getpwuid()
+#include <grp.h>	// getgrgid()
 #include <unistd.h>
 
 #include <QDateTime>
@@ -42,6 +44,8 @@ FileInfo::FileInfo( DirTree    * tree,
     _device	  = 0;
     _mode	  = 0;
     _links	  = 0;
+    _uid	  = 0;
+    _gid	  = 0;
     _size	  = 0;
     _blocks	  = 0;
     _mtime	  = 0;
@@ -65,6 +69,8 @@ FileInfo::FileInfo( const QString & filenameWithoutPath,
     _device	 = statInfo->st_dev;
     _mode	 = statInfo->st_mode;
     _links	 = statInfo->st_nlink;
+    _uid	 = statInfo->st_uid;
+    _gid	 = statInfo->st_gid;
     _mtime	 = statInfo->st_mtime;
     _magic	 = FileInfoMagic;
 
@@ -116,11 +122,13 @@ FileInfo::FileInfo( DirTree *	    tree,
 {
     _name	 = filenameWithoutPath;
     _isLocalFile = true;
-    _device      = 0;
+    _device	 = 0;
     _mode	 = mode;
     _size	 = size;
     _mtime	 = mtime;
     _links	 = links;
+    _uid	 = 0;
+    _gid	 = 0;
     _magic	 = FileInfoMagic;
 
     if ( blocks < 0 )
@@ -331,6 +339,74 @@ QString FileInfo::dotEntryName()
 }
 
 
+bool FileInfo::isCached() const
+{
+    if ( isDirInfo() && ! isDotEntry() )
+        return readState() == DirCached;
+    else
+        return _parent && _parent->readState() == DirCached;
+}
+
+
+bool FileInfo::hasUid() const
+{
+    return ! isCached();
+}
+
+
+bool FileInfo::hasGid() const
+{
+    return ! isCached();
+}
+
+
+QString FileInfo::userName() const
+{
+    if ( ! hasUid() )
+	return QString();
+
+    struct passwd * pw = getpwuid( uid() );
+
+    if ( pw )
+	return pw->pw_name;
+    else
+	return QString::number( uid() );
+}
+
+
+QString FileInfo::groupName() const
+{
+    if ( ! hasGid() )
+	return QString();
+
+    struct group * grp = getgrgid( gid() );
+
+    if ( grp )
+	return grp->gr_name;
+    else
+	return QString::number( gid() );
+}
+
+
+QString FileInfo::symbolicPermissions() const
+{
+    return symbolicMode( _mode,
+			 true ); // omitTypeForRegularFiles
+}
+
+
+QString FileInfo::octalPermissions() const
+{
+    return formatOctal( ALLPERMS & _mode );
+}
+
+
+QString FileInfo::baseName() const
+{
+    return QDirStat::baseName( _name );
+}
+
+
 //---------------------------------------------------------------------------
 
 
@@ -395,9 +471,82 @@ QString QDirStat::formatTime( time_t rawTime )
 }
 
 
+// See also FileInfo::baseName()
+
 QString QDirStat::baseName( const QString & fileName )
 {
     QStringList segments = fileName.split( '/', QString::SkipEmptyParts );
     return segments.isEmpty() ? "" : segments.last();
 }
 
+
+QString QDirStat::formatPermissions( mode_t mode )
+{
+    return symbolicMode( mode, true ) + "  " + formatOctal( ALLPERMS & mode );
+}
+
+
+QString QDirStat::formatOctal( int number )
+{
+    return QString( "0" ) + QString::number( number, 8 );
+}
+
+
+QString QDirStat::symbolicMode( mode_t mode, bool omitTypeForRegularFiles )
+{
+    QString result;
+
+    // Type
+
+    if	    ( S_ISDIR ( mode ) )	  result = "d";
+    else if ( S_ISCHR ( mode ) )	  result = "c";
+    else if ( S_ISBLK ( mode ) )	  result = "b";
+    else if ( S_ISFIFO( mode ) )	  result = "p";
+    else if ( S_ISLNK ( mode ) )	  result = "l";
+    else if ( S_ISSOCK( mode ) )	  result = "s";
+    else if ( ! omitTypeForRegularFiles ) result = "-";
+
+    // User
+
+    result += ( mode & S_IRUSR ) ? "r" : "-";
+    result += ( mode & S_IWUSR ) ? "w" : "-";
+
+    if ( mode & S_ISUID )
+	result += "s";
+    else
+	result += ( mode & S_IXUSR ) ? "x" : "-";
+
+    // Group
+
+    result += ( mode & S_IRGRP ) ? "r" : "-";
+    result += ( mode & S_IWGRP ) ? "w" : "-";
+
+    if ( mode & S_ISGID )
+	result += "s";
+    else
+	result += ( mode & S_IXGRP ) ? "x" : "-";
+
+    // Other
+
+    result += ( mode & S_IROTH ) ? "r" : "-";
+    result += ( mode & S_IWOTH ) ? "w" : "-";
+
+    if ( mode & S_ISVTX )
+	result += "t";
+    else
+	result += ( mode & S_IXOTH ) ? "x" : "-";
+
+    return result;
+}
+
+
+QString QDirStat::formatFileSystemObjectType( mode_t mode )
+{
+    if	    ( S_ISDIR ( mode ) ) return QObject::tr( "Directory"	);
+    else if ( S_ISCHR ( mode ) ) return QObject::tr( "Character Device" );
+    else if ( S_ISBLK ( mode ) ) return QObject::tr( "Block Device"	);
+    else if ( S_ISFIFO( mode ) ) return QObject::tr( "Named Pipe"	);
+    else if ( S_ISLNK ( mode ) ) return QObject::tr( "Symbolic Link"	);
+    else if ( S_ISSOCK( mode ) ) return QObject::tr( "Socket"		);
+    else			 return QObject::tr( "File"		);
+}
