@@ -134,6 +134,12 @@ namespace QDirStat
 	 **/
 	QString name() const { return _name; }
 
+        /**
+         * Returns the base name of this object, i.e. the last path component,
+         * even if this is a toplevel item.
+         **/
+        QString baseName() const;
+
 	/**
 	 * Returns the full URL of this object with full path.
 	 *
@@ -162,6 +168,8 @@ namespace QDirStat
 	 * The file permissions and object type as returned by lstat().
 	 * You might want to use the repective convenience methods instead:
 	 * @ref isDir(), @ref isFile(), ...
+         *
+         * See also @ref symbolicPermissions(), octalPermissions()
 	 **/
 	mode_t mode() const { return _mode;   }
 
@@ -171,6 +179,66 @@ namespace QDirStat
 	 **/
 	nlink_t links() const { return _links;	}
 
+	/**
+	 * User ID of the owner.
+	 *
+	 * Notice that this might be undefined if this tree branch was read
+	 * from a cache file. Check that with hasUid().
+	 **/
+	uid_t uid() const { return _uid; }
+
+	/**
+	 * Return the user name of the owner.
+	 *
+	 * If this tree branch was read from a cache file, this returns an
+	 * empty string.
+	 **/
+	QString userName() const;
+
+        /**
+         * Return 'true' if this FileInfo has a UID (user ID).x
+         *
+         * It might not have that information e.g. if it was read from a cache
+         * file.
+         **/
+        bool hasUid() const;
+
+	/**
+	 * Group ID of the owner.
+	 *
+	 * Notice that this might be undefined if this tree branch was read
+	 * from a cache file. Check that with hasGid().
+	 **/
+	gid_t gid() const { return _gid; }
+
+	/**
+	 * Return the group name of the owner.
+	 *
+	 * If this tree branch was read from a cache file, this returns an
+	 * empty string.
+	 **/
+	QString groupName() const;
+
+        /**
+         * Return 'true' if this FileInfo has a GID (group ID).
+         *
+         * It might not have that information e.g. if it was read from a cache
+         * file.
+         **/
+        bool hasGid() const;
+
+        /**
+         * File permissions formatted like in "ls -l", i.e. "-rwxrwxrwx",
+         * "drwxrwxrwx"
+         **/
+        QString symbolicPermissions() const;
+
+        /**
+         * File permissions formatted as octal number (like used by the "chmod"
+         * command, i.e. "0644"
+         **/
+        QString octalPermissions() const;
+
 	/**
 	 * The file size in bytes. This does not take unused space in the last
 	 * disk block (cluster) into account, yet it is the only size all kinds
@@ -495,7 +563,8 @@ namespace QDirStat
 	 * know - better be safe than sorry!
 	 *
 	 * This default implementation always returns 'false'. Derived classes
-	 * (in particular, those derived from @ref DirInfo) should overwrite this.
+	 * (in particular, those derived from @ref DirInfo) should overwrite
+	 * this.
 	 **/
 	virtual bool isDirInfo() const { return false; }
 
@@ -518,6 +587,11 @@ namespace QDirStat
 	 **/
 	bool isSparseFile() const { return _isSparseFile; }
 
+        /**
+         * Returns true if this FileInfo was read from a cache file.
+         **/
+        bool isCached() const;
+
 
 	//
 	// File type / mode convenience methods.
@@ -590,11 +664,13 @@ namespace QDirStat
 	dev_t		_device;		// device this object resides on
 	mode_t		_mode;			// file permissions + object type
 	nlink_t		_links;			// number of links
+	uid_t		_uid;			// User ID of owner
+	gid_t		_gid;			// Group ID of owner
 	FileSize	_size;			// size in bytes
 	FileSize	_blocks;		// 512 bytes blocks
 	time_t		_mtime;			// modification time
 
-	DirInfo  *	_parent;		// pointer to the parent entry
+	DirInfo	 *	_parent;		// pointer to the parent entry
 	FileInfo *	_next;			// pointer to the next entry
 	DirTree	 *	_tree;			// pointer to the parent tree
 
@@ -635,9 +711,39 @@ namespace QDirStat
      *	   "foo.txt"		-> "foo.txt"
      *	   "/usr/bin"		-> "bin"
      *	   "/usr/bin/"		-> "bin"
+     *
+     * Notice that FileInfo also has a member function baseName().
      **/
     QString baseName( const QString & fileName );
 
+    /**
+     * Format the mode (the permissions bits) returned from the stat() system
+     * call in the commonly used formats, both symbolic and octal, e.g.
+     *     drwxr-xr-x  0755
+     **/
+    QString formatPermissions( mode_t mode );
+
+    /**
+     * Format a number in octal with a leading zero.
+     **/
+    QString formatOctal( int number );
+
+    /**
+     * Return the mode (the permission bits) returned from stat() like the
+     * "ls -l" shell command does, e.g.
+     *
+     *     drwxr-xr-x
+     *
+     * 'omitTypeForRegularFiles' specifies if the leading "-" should be omitted.
+     **/
+    QString symbolicMode( mode_t perm, bool omitTypeForRegularFiles = false );
+
+    /**
+     * Format the filesystem object type from a mode, e.g. "Directory",
+     * "Symbolic Link", "Block Device", "File".
+     **/
+    QString formatFileSystemObjectType( mode_t mode );
+
 
     /**
      * Print the debugUrl() of a @ref FileInfo in a debug stream.
