@@ -27,8 +27,10 @@
 #include "DirTreeModel.h"
 #include "Exception.h"
 #include "ExcludeRules.h"
+#include "FileDetailsView.h"
 #include "FileInfo.h"
 #include "FileSizeStatsWindow.h"
+#include "HeaderTweaker.h"
 #include "Logger.h"
 #include "MimeCategorizer.h"
 #include "MimeCategoryConfigPage.h"
@@ -37,9 +39,13 @@
 #include "SelectionModel.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
+#include "SysUtil.h"
 #include "Trash.h"
 #include "Version.h"
 
+#define LONG_MESSAGE	25*1000
+#define UPDATE_MILLISEC 200
+
 using namespace QDirStat;
 
 using QDirStat::DataColumns;
@@ -54,14 +60,21 @@ MainWindow::MainWindow():
     _configDialog(0),
     _modified( false ),
     _verboseSelection( false ),
+    _urlInWindowTitle( false ),
+    _useTreemapHover( false ),
     _statusBarTimeout( 3000 ), // millisec
-    _treeLevelMapper(0)
+    _treeLevelMapper(0),
+    _currentLayout( 0 )
 {
     CHECK_PTR( _ui );
 
     _ui->setupUi( this );
     ActionManager::instance()->addWidgetTree( this );
+    initLayoutActions();
+    createLayouts();
     readSettings();
+    _updateTimer.setInterval( UPDATE_MILLISEC );
+    _dUrl = _ui->actionDonate->iconText();
 
     _dirTreeModel = new DirTreeModel( this );
     CHECK_NEW( _dirTreeModel );
@@ -76,6 +89,7 @@ MainWindow::MainWindow():
     _ui->treemapView->setSelectionModel( _selectionModel );
 
     _dirTreeModel->setSelectionModel( _selectionModel );
+    changeLayout( _layoutName );
 
     _cleanupCollection = new CleanupCollection( _selectionModel );
     CHECK_NEW( _cleanupCollection );
@@ -95,68 +109,14 @@ MainWindow::MainWindow():
     _ui->treemapView->setMimeCategorizer( _mimeCategorizer );
 
 #ifdef Q_OS_MACX
-    // this makes the application to look like more "native" on macOS
+    // This makes the application to look like more "native" on macOS
     setUnifiedTitleAndToolBarOnMac(true);
     _ui->toolBar->setMovable(false);
 #endif
 
-    connect( _selectionModel,  SIGNAL( currentBranchChanged( QModelIndex ) ),
-	     _ui->dirTreeView, SLOT  ( closeAllExcept	   ( QModelIndex ) ) );
-
-    connect( _dirTreeModel->tree(),	SIGNAL( startingReading() ),
-	     this,			SLOT  ( startingReading() ) );
-
-    connect( _dirTreeModel->tree(),	SIGNAL( finished()	  ),
-	     this,			SLOT  ( readingFinished() ) );
-
-    connect( _dirTreeModel->tree(),	SIGNAL( aborted()	  ),
-	     this,			SLOT  ( readingAborted()  ) );
-
-    connect( _selectionModel,  SIGNAL( selectionChanged() ),
-	     this,	       SLOT  ( updateActions()	 ) );
-
-    connect( _selectionModel,  SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
-	     this,	       SLOT  ( updateActions()				   ) );
-
-    connect( _ui->treemapView, SIGNAL( treemapChanged() ),
-	     this,	       SLOT  ( updateActions()	 ) );
-
-    connect( _cleanupCollection, SIGNAL( startingCleanup( QString ) ),
-	     this,		 SLOT  ( startingCleanup( QString ) ) );
-
-    connect( _cleanupCollection, SIGNAL( cleanupFinished( int ) ),
-	     this,		 SLOT  ( cleanupFinished( int ) ) );
-
-    connect( _ui->treemapView,   SIGNAL( hoverEnter ( FileInfo * ) ),
-             this,               SLOT  ( showCurrent( FileInfo * ) ) );
-
-    connect( _ui->treemapView,   SIGNAL( hoverLeave ( FileInfo * ) ),
-             this,               SLOT  ( showSummary()             ) );
-
-
-    // Debug connections
-
-    connect( _ui->dirTreeView, SIGNAL( clicked	  ( QModelIndex ) ),
-	     this,	       SLOT  ( itemClicked( QModelIndex ) ) );
-
-    connect( _selectionModel, SIGNAL( selectionChanged() ),
-	     this,	      SLOT  ( selectionChanged() ) );
-
-    connect( _selectionModel, SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
-	     this,	      SLOT  ( currentItemChanged( FileInfo *, FileInfo * ) ) );
-
+    connectSignals();
     connectActions();
 
-
-    if ( ExcludeRules::instance()->isEmpty() )
-    {
-	ExcludeRules::add( QRegExp( ".snapshot",
-				    Qt::CaseSensitive,
-				    QRegExp::FixedString ) );
-    }
-
-    Debug::dumpExcludeRules();
-
     if ( ! _ui->actionShowTreemap->isChecked() )
 	_ui->treemapView->disable();
 
@@ -167,6 +127,9 @@ MainWindow::MainWindow():
 
 MainWindow::~MainWindow()
 {
+    if ( _currentLayout )
+	saveLayout( _currentLayout );
+
     writeSettings();
     ExcludeRules::instance()->writeSettings();
 
@@ -181,6 +144,68 @@ MainWindow::~MainWindow()
     delete _mimeCategorizer;
     delete _selectionModel;
     delete _dirTreeModel;
+
+    qDeleteAll( _layouts );
+}
+
+
+void MainWindow::connectSignals()
+{
+    connect( _selectionModel,		SIGNAL( currentBranchChanged( QModelIndex ) ),
+	     _ui->dirTreeView,		SLOT  ( closeAllExcept	    ( QModelIndex ) ) );
+
+    connect( _dirTreeModel->tree(),	SIGNAL( startingReading() ),
+	     this,			SLOT  ( startingReading() ) );
+
+    connect( _dirTreeModel->tree(),	SIGNAL( finished()	  ),
+	     this,			SLOT  ( readingFinished() ) );
+
+    connect( _dirTreeModel->tree(),	SIGNAL( aborted()	  ),
+	     this,			SLOT  ( readingAborted()  ) );
+
+    connect( _selectionModel,		SIGNAL( selectionChanged() ),
+	     this,			SLOT  ( updateActions()	   ) );
+
+    connect( _selectionModel,		SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
+	     this,			SLOT  ( updateActions()				    ) );
+
+    connect( _selectionModel,		SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
+	     _ui->breadcrumbNavigator,	SLOT  ( setPath		  ( FileInfo *		   ) ) );
+
+    connect( _ui->breadcrumbNavigator,	SIGNAL( pathClicked   ( QString ) ),
+	     _selectionModel,		SLOT  ( setCurrentItem( QString ) ) );
+
+    connect( _ui->treemapView,		SIGNAL( treemapChanged() ),
+	     this,			SLOT  ( updateActions()	 ) );
+
+    connect( _cleanupCollection,	SIGNAL( startingCleanup( QString ) ),
+	     this,			SLOT  ( startingCleanup( QString ) ) );
+
+    connect( _cleanupCollection,	SIGNAL( cleanupFinished( int ) ),
+	     this,			SLOT  ( cleanupFinished( int ) ) );
+
+    connect( &_updateTimer,		SIGNAL( timeout()	  ),
+	     this,			SLOT  ( showElapsedTime() ) );
+
+    if ( _useTreemapHover )
+    {
+	connect( _ui->treemapView,	SIGNAL( hoverEnter ( FileInfo * ) ),
+		 this,			SLOT  ( showCurrent( FileInfo * ) ) );
+
+	connect( _ui->treemapView,	SIGNAL( hoverLeave ( FileInfo * ) ),
+		 this,			SLOT  ( showSummary()		) );
+    }
+
+    // Debug connections
+
+    connect( _ui->dirTreeView,		SIGNAL( clicked	   ( QModelIndex ) ),
+	     this,			SLOT  ( itemClicked( QModelIndex ) ) );
+
+    connect( _selectionModel,		SIGNAL( selectionChanged() ),
+	     this,			SLOT  ( selectionChanged() ) );
+
+    connect( _selectionModel,		SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
+	     this,			SLOT  ( currentItemChanged( FileInfo *, FileInfo * ) ) );
 }
 
 
@@ -224,6 +249,16 @@ void MainWindow::connectActions()
 
     mapTreeExpandAction( _ui->actionCloseAllTreeLevels, 0 );
 
+    connect( _ui->actionShowCurrentPath,  SIGNAL( toggled   ( bool ) ),
+	     _ui->breadcrumbNavigator,	  SLOT	( setVisible( bool ) ) );
+
+    connect( _ui->actionShowDetailsPanel, SIGNAL( toggled   ( bool ) ),
+	     _ui->fileDetailsPanel,	  SLOT	( setVisible( bool ) ) );
+
+    CONNECT_ACTION( _ui->actionLayout1,		   this, changeLayout() );
+    CONNECT_ACTION( _ui->actionLayout2,		   this, changeLayout() );
+    CONNECT_ACTION( _ui->actionLayout3,		   this, changeLayout() );
+
     CONNECT_ACTION( _ui->actionFileSizeStats,	   this, showFileSizeStats() );
     CONNECT_ACTION( _ui->actionFileTypeStats,	   this, showFileTypeStats() );
 
@@ -248,8 +283,8 @@ void MainWindow::connectActions()
     connect( _ui->actionShowTreemap, SIGNAL( toggled( bool )   ),
 	     this,		     SLOT  ( showTreemapView() ) );
 
-    connect( _ui->actionTreemapAsSidePanel, SIGNAL( toggled( bool )      ),
-             this,		            SLOT  ( treemapAsSidePanel() ) );
+    connect( _ui->actionTreemapAsSidePanel, SIGNAL( toggled( bool )	 ),
+	     this,			    SLOT  ( treemapAsSidePanel() ) );
 
     CONNECT_ACTION( _ui->actionTreemapZoomIn,	 _ui->treemapView, zoomIn()	    );
     CONNECT_ACTION( _ui->actionTreemapZoomOut,	 _ui->treemapView, zoomOut()	    );
@@ -263,9 +298,11 @@ void MainWindow::connectActions()
 
     // "Help" menu
 
-    CONNECT_ACTION( _ui->actionHelp,	this, showHelp() );
-    CONNECT_ACTION( _ui->actionAbout,	this, showAboutDialog() );
-    CONNECT_ACTION( _ui->actionAboutQt, qApp, aboutQt() );
+    CONNECT_ACTION( _ui->actionHelp,	    this, showHelp() );
+    CONNECT_ACTION( _ui->actionAbout,	    this, showAboutDialog() );
+    CONNECT_ACTION( _ui->actionAboutQt,	    qApp, aboutQt() );
+    CONNECT_ACTION( _ui->actionWhatsNew,    this, showWhatsNew() );
+    CONNECT_ACTION( _ui->actionDonate,	    this, showDonateDialog() );
 
 
     // Invisible debug actions
@@ -307,13 +344,13 @@ void MainWindow::updateActions()
     FileInfo * sel = selectedItems.first();
     bool oneDirSelected = ( selectedItems.size() == 1 ) && sel && sel->isDir();
 
-    _ui->actionMoveToTrash->setEnabled( sel && ! sel->isDotEntry() );
+    _ui->actionMoveToTrash->setEnabled( sel && ! sel->isDotEntry() && ! reading );
     _ui->actionRefreshSelected->setEnabled( oneDirSelected && ! sel->isExcluded() && ! sel->isMountPoint() );
     _ui->actionContinueReadingAtMountPoint->setEnabled( oneDirSelected && sel->isMountPoint() );
     _ui->actionReadExcludedDirectory->setEnabled      ( oneDirSelected && sel->isExcluded()   );
 
     bool nothingOrOneDir = selectedItems.isEmpty() ||
-        ( selectedItems.size() == 1 && sel && ( sel->isDir() || sel->isDotEntry() ) );
+	( selectedItems.size() == 1 && sel && ( sel->isDir() || sel->isDotEntry() ) );
 
     _ui->actionFileSizeStats->setEnabled( ! reading && nothingOrOneDir );
     _ui->actionFileTypeStats->setEnabled( ! reading && nothingOrOneDir );
@@ -333,11 +370,20 @@ void MainWindow::readSettings()
     QDirStat::Settings settings;
     settings.beginGroup( "MainWindow" );
 
-    _statusBarTimeout  = settings.value( "StatusBarTimeoutMillisec", 3000  ).toInt();
-    bool showTreemap   = settings.value( "ShowTreemap"	           , true  ).toBool();
-    bool treemapOnSide = settings.value( "TreemapOnSide"	   , false ).toBool();
-    _verboseSelection  = settings.value( "VerboseSelection"	   , false ).toBool();
+    _statusBarTimeout	  = settings.value( "StatusBarTimeoutMillisec", 3000  ).toInt();
+    bool showTreemap	  = settings.value( "ShowTreemap"	      , true  ).toBool();
+    bool treemapOnSide	  = settings.value( "TreemapOnSide"	      , false ).toBool();
 
+    _verboseSelection	  = settings.value( "VerboseSelection"	      , false ).toBool();
+    _urlInWindowTitle	  = settings.value( "UrlInWindowTitle"	      , false ).toBool();
+    _useTreemapHover	  = settings.value( "UseTreemapHover"	      , false ).toBool();
+    _layoutName		  = settings.value( "Layout"		      , "L2"  ).toString();
+
+    settings.endGroup();
+
+    settings.beginGroup( "MainWindow-Subwindows" );
+    QByteArray mainSplitterState = settings.value( "MainSplitter" , QByteArray() ).toByteArray();
+    QByteArray topSplitterState	 = settings.value( "TopSplitter"  , QByteArray() ).toByteArray();
     settings.endGroup();
 
     _ui->actionShowTreemap->setChecked( showTreemap );
@@ -346,8 +392,47 @@ void MainWindow::readSettings()
 
     _ui->actionVerboseSelection->setChecked( _verboseSelection );
 
+    foreach ( QAction * action, _layoutActionGroup->actions() )
+    {
+	if ( action->data().toString() == _layoutName )
+	    action->setChecked( true );
+    }
+
     readWindowSettings( this, "MainWindow" );
+
+    if ( ! mainSplitterState.isNull() )
+	_ui->mainWinSplitter->restoreState( mainSplitterState );
+
+    if ( ! topSplitterState.isNull() )
+	_ui->topViewsSplitter->restoreState( topSplitterState );
+    else
+    {
+	// The Qt designer refuses to let me set a reasonable size for that
+	// widget, so let's set one here. Yes, that's not really how this is
+	// supposed to be, but I am fed up with that stuff.
+
+	_ui->fileDetailsPanel->resize( QSize( 300, 300 ) );
+    }
+
+    foreach ( TreeLayout * layout, _layouts )
+	readLayoutSettings( layout );
+
     ExcludeRules::instance()->readSettings();
+    Debug::dumpExcludeRules();
+}
+
+
+void MainWindow::readLayoutSettings( TreeLayout * layout )
+{
+    CHECK_PTR( layout );
+
+    Settings settings;
+    settings.beginGroup( QString( "TreeViewLayout_%1" ).arg( layout->name ) );
+
+    layout->showCurrentPath  = settings.value( "ShowCurrentPath" , layout->showCurrentPath  ).toBool();
+    layout->showDetailsPanel = settings.value( "ShowDetailsPanel", layout->showDetailsPanel ).toBool();
+
+    settings.endGroup();
 }
 
 
@@ -356,14 +441,42 @@ void MainWindow::writeSettings()
     QDirStat::Settings settings;
     settings.beginGroup( "MainWindow" );
 
-    settings.setValue( "StatusBarTimeoutMillisec", _statusBarTimeout );
-    settings.setValue( "ShowTreemap"		 , _ui->actionShowTreemap->isChecked() );
-    settings.setValue( "TreemapOnSide"		 , _ui->actionTreemapAsSidePanel->isChecked() );
-    settings.setValue( "VerboseSelection"	 , _verboseSelection );
+    settings.setValue( "ShowTreemap"	 , _ui->actionShowTreemap->isChecked() );
+    settings.setValue( "TreemapOnSide"	 , _ui->actionTreemapAsSidePanel->isChecked() );
+    settings.setValue( "VerboseSelection", _verboseSelection );
+    settings.setValue( "Layout"		 , _layoutName );
+
+    // Those are only set if not already in the settings (they might have been
+    // set from a config dialog).
+    settings.setDefaultValue( "StatusBarTimeoutMillisec", _statusBarTimeout );
+    settings.setDefaultValue( "UrlInWindowTitle"	, _urlInWindowTitle );
+    settings.setDefaultValue( "UseTreemapHover"		, _useTreemapHover );
 
     settings.endGroup();
 
     writeWindowSettings( this, "MainWindow" );
+
+    settings.beginGroup( "MainWindow-Subwindows" );
+    settings.setValue( "MainSplitter"		 , _ui->mainWinSplitter->saveState()  );
+    settings.setValue( "TopSplitter"		 , _ui->topViewsSplitter->saveState() );
+    settings.endGroup();
+
+    foreach ( TreeLayout * layout, _layouts )
+	writeLayoutSettings( layout );
+}
+
+
+void MainWindow::writeLayoutSettings( TreeLayout * layout )
+{
+    CHECK_PTR( layout );
+
+    Settings settings;
+    settings.beginGroup( QString( "TreeViewLayout_%1" ).arg( layout->name ) );
+
+    settings.setValue( "ShowCurrentPath" , layout->showCurrentPath  );
+    settings.setValue( "ShowDetailsPanel", layout->showDetailsPanel );
+
+    settings.endGroup();
 }
 
 
@@ -379,9 +492,9 @@ void MainWindow::showTreemapView()
 void MainWindow::treemapAsSidePanel()
 {
     if ( _ui->actionTreemapAsSidePanel->isChecked() )
-        _ui->mainWinSplitter->setOrientation(Qt::Horizontal);
+	_ui->mainWinSplitter->setOrientation(Qt::Horizontal);
     else
-        _ui->mainWinSplitter->setOrientation(Qt::Vertical);
+	_ui->mainWinSplitter->setOrientation(Qt::Vertical);
 }
 
 
@@ -419,6 +532,7 @@ void MainWindow::busyDisplay()
 {
     _ui->treemapView->disable();
     updateActions();
+    _updateTimer.start();
 
     // It would be nice to sort by read jobs during reading, but this confuses
     // the hell out of the Qt side of the data model; so let's sort by name
@@ -438,6 +552,7 @@ void MainWindow::busyDisplay()
 void MainWindow::idleDisplay()
 {
     updateActions();
+    _updateTimer.stop();
     int sortCol = QDirStat::DataColumns::toViewCol( QDirStat::PercentNumCol );
     _ui->dirTreeView->sortByColumn( sortCol, Qt::DescendingOrder );
 
@@ -447,6 +562,7 @@ void MainWindow::idleDisplay()
 	expandTreeToLevel( 1 );
     }
 
+    updateFileDetailsView();
     showTreemapView();
 }
 
@@ -463,8 +579,10 @@ void MainWindow::readingFinished()
     logInfo() << endl;
 
     idleDisplay();
-    _ui->statusBar->showMessage( tr( "Finished. Elapsed time: %1")
-				 .arg( formatTime( _stopWatch.elapsed() ) ) );
+
+    QString elapsedTime = formatTime( _stopWatch.elapsed() );
+    _ui->statusBar->showMessage( tr( "Finished. Elapsed time: %1").arg( elapsedTime ), LONG_MESSAGE );
+    logInfo() << "Reading finished after " << elapsedTime << endl;
 
     // Debug::dumpModelTree( _dirTreeModel, QModelIndex(), "" );
 }
@@ -475,29 +593,47 @@ void MainWindow::readingAborted()
     logInfo() << endl;
 
     idleDisplay();
-    _ui->statusBar->showMessage( tr( "Aborted. Elapsed time: %1")
-				 .arg( formatTime( _stopWatch.elapsed() ) ) );
+    QString elapsedTime = formatTime( _stopWatch.elapsed() );
+    _ui->statusBar->showMessage( tr( "Aborted. Elapsed time: %1").arg( elapsedTime ), LONG_MESSAGE );
+    logInfo() << "Reading aborted after " << elapsedTime << endl;
 }
 
 
 void MainWindow::openUrl( const QString & url )
 {
+    QString windowTitle = "QDirStat";
+
+    if ( SysUtil::runningAsRoot() )
+	windowTitle += tr( " [root]" );
+
     try
     {
 	_dirTreeModel->openUrl( url );
+
+	if ( _urlInWindowTitle )
+	{
+	    // The URL we got at the command line might be relative;
+	    // the DirTree converts this into an absolute URL.
+	    // Let's use that.
+
+	    windowTitle += " " + _dirTreeModel->tree()->url();
+	}
+
+	setWindowTitle( windowTitle );
     }
     catch ( const SysCallFailedException & ex )
     {
-        CAUGHT( ex );
+	setWindowTitle( windowTitle );
+	CAUGHT( ex );
 
-	QMessageBox errorPopup( QMessageBox::Warning,   // icon
-                                tr( "Error" ),		// title
-                                tr( "Could not open directory %1" ).arg( ex.resourceName() ), // text
-                                QMessageBox::Ok,	// buttons
-                                this );                 // parent
+	QMessageBox errorPopup( QMessageBox::Warning,	// icon
+				tr( "Error" ),		// title
+				tr( "Could not open directory %1" ).arg( ex.resourceName() ), // text
+				QMessageBox::Ok,	// buttons
+				this );			// parent
 	errorPopup.setDetailedText( ex.what() );
 	errorPopup.exec();
-        askOpenUrl();
+	askOpenUrl();
     }
 
     updateActions();
@@ -544,7 +680,7 @@ void MainWindow::stopReading()
     if ( _dirTreeModel->tree()->isBusy() )
     {
 	_dirTreeModel->tree()->abortReading();
-	_ui->statusBar->showMessage( tr( "Reading aborted." ) );
+	_ui->statusBar->showMessage( tr( "Reading aborted." ), LONG_MESSAGE );
     }
 }
 
@@ -577,9 +713,16 @@ void MainWindow::askWriteCache()
     {
 	bool ok = _dirTreeModel->tree()->writeCache( fileName );
 
-	QString msg = ok ? tr( "Directory tree written to file %1" ).arg( fileName ) :
-			   tr( "ERROR writing cache file %1").arg( fileName );
-	_ui->statusBar->showMessage( msg, _statusBarTimeout );
+	if ( ok )
+	{
+	    showProgress( tr( "Directory tree written to file %1" ).arg( fileName ) );
+	}
+	else
+	{
+	    QMessageBox::critical( this,
+				   tr( "Error" ), // Title
+				   tr( "ERROR writing cache file %1").arg( fileName ) );
+	}
     }
 }
 
@@ -634,6 +777,25 @@ void MainWindow::showSummary()
 }
 
 
+void MainWindow::updateFileDetailsView()
+{
+    if ( _ui->fileDetailsView->isVisible() )
+    {
+	FileInfoSet sel = _selectionModel->selectedItems();
+
+	if ( sel.isEmpty() )
+	    _ui->fileDetailsView->showDetails( _selectionModel->currentItem() );
+	else
+	{
+	    if ( sel.count() == 1 )
+		_ui->fileDetailsView->showDetails( sel.first() );
+	    else
+		_ui->fileDetailsView->showDetails( sel );
+	}
+    }
+}
+
+
 void MainWindow::startingCleanup( const QString & cleanupName )
 {
     showProgress( tr( "Starting cleanup action %1" ).arg( cleanupName ) );
@@ -767,15 +929,22 @@ void MainWindow::openConfigDialog()
 }
 
 
+void MainWindow::showElapsedTime()
+{
+    showProgress( tr( "Reading... %1" )
+		  .arg( formatTime( _stopWatch.elapsed(), false ) ) );
+}
+
+
 void MainWindow::showFileTypeStats()
 {
     if ( ! _fileTypeStatsWindow )
     {
-        // This deletes itself when the user closes it. The associated QPointer
-        // keeps track of that and sets the pointer to 0 when it happens.
+	// This deletes itself when the user closes it. The associated QPointer
+	// keeps track of that and sets the pointer to 0 when it happens.
 
-        _fileTypeStatsWindow = new QDirStat::FileTypeStatsWindow( _selectionModel,
-                                                                  this );
+	_fileTypeStatsWindow = new QDirStat::FileTypeStatsWindow( _selectionModel,
+								  this );
     }
 
     _fileTypeStatsWindow->populate( selectedDirOrRoot() );
@@ -788,19 +957,107 @@ void MainWindow::showFileSizeStats()
     FileInfo * sel = selectedDirOrRoot();
 
     if ( ! sel || ! sel->hasChildren() )
-        return;
+	return;
 
     FileSizeStatsWindow::populateSharedInstance( sel );
 }
 
 
+void MainWindow::initLayoutActions()
+{
+    // Qt Designer does not support QActionGroups; it was there for Qt 3, but
+    // they dropped that feature for Qt 4/5.
+
+    _layoutActionGroup = new QActionGroup( this );
+    CHECK_NEW( _layoutActionGroup );
+
+    _layoutActionGroup->addAction( _ui->actionLayout1 );
+    _layoutActionGroup->addAction( _ui->actionLayout2 );
+    _layoutActionGroup->addAction( _ui->actionLayout3 );
+
+    _ui->actionLayout1->setData( "L1" );
+    _ui->actionLayout2->setData( "L2" );
+    _ui->actionLayout3->setData( "L3" );
+}
+
+
+void MainWindow::createLayouts()
+{
+    TreeLayout * layout;
+
+    layout = new TreeLayout( "L1" );
+    CHECK_NEW( layout );
+    _layouts[ "L1" ] = layout;
+
+    layout = new TreeLayout( "L2" );
+    CHECK_NEW( layout );
+    _layouts[ "L2" ] = layout;
+
+    layout = new TreeLayout( "L3" );
+    CHECK_NEW( layout );
+    _layouts[ "L3" ] = layout;
+
+    layout->showDetailsPanel = false;
+}
+
+
+void MainWindow::changeLayout( const QString & name )
+{
+    _layoutName = name;
+
+    if ( _layoutName.isEmpty() )
+    {
+	// Get the layout to use from data() from the QAction that sent the signal.
+
+	QAction * action   = qobject_cast<QAction *>( sender() );
+	_layoutName = action && action->data().isValid() ?
+	    action->data().toString() : "L2";
+    }
+
+    logDebug() << "Changing to layout " << _layoutName << endl;
+
+    _ui->dirTreeView->headerTweaker()->changeLayout( _layoutName );
+
+    if ( _currentLayout )
+	saveLayout( _currentLayout );
+
+    if ( _layouts.contains( _layoutName ) )
+    {
+	_currentLayout = _layouts[ _layoutName ];
+	applyLayout( _currentLayout );
+    }
+    else
+    {
+	logError() << "No layout " << _layoutName << endl;
+    }
+}
+
+
+void MainWindow::saveLayout( TreeLayout * layout )
+{
+    CHECK_PTR( layout );
+
+    layout->showCurrentPath  = _ui->actionShowCurrentPath->isChecked();
+    layout->showDetailsPanel = _ui->actionShowDetailsPanel->isChecked();
+}
+
+
+void MainWindow::applyLayout( TreeLayout * layout )
+{
+    CHECK_PTR( layout );
+
+    _ui->actionShowCurrentPath->setChecked ( layout->showCurrentPath  );
+    _ui->actionShowDetailsPanel->setChecked( layout->showDetailsPanel );
+}
+
+
 FileInfo * MainWindow::selectedDirOrRoot() const
 {
     FileInfoSet selectedItems = _selectionModel->selectedItems();
     FileInfo * sel = selectedItems.first();
 
     if ( ! sel )
-        sel = _dirTreeModel->tree()->firstToplevel();
+	sel = _dirTreeModel->tree()->firstToplevel();
 
     return sel;
 }
@@ -822,10 +1079,13 @@ void MainWindow::toggleVerboseSelection()
 
 void MainWindow::showHelp()
 {
-    QString helpUrl = "https://github.com/shundhammer/qdirstat/blob/master/README.md";
-    QString program = "/usr/bin/xdg-open";
-    logInfo() << "Starting " << program << " " << helpUrl << endl;
-    QProcess::startDetached( program, QStringList() << helpUrl );
+    SysUtil::openInBrowser( "https://github.com/shundhammer/qdirstat/blob/master/README.md" );
+}
+
+
+void MainWindow::showWhatsNew()
+{
+    SysUtil::openInBrowser( RELEASE_URL ); // defined in Version.h
 }
 
 
@@ -839,12 +1099,12 @@ void MainWindow::showAboutDialog()
     text += tr( "Qt-based directory statistics -- showing where all your disk space has gone "
 		" and trying to help you to clean it up." );
     text += "</p><p>";
-    text += "(c) 2015-2017 Stefan Hundhammer";
+    text += "(c) 2015-2018 Stefan Hundhammer";
     text += "</p><p>";
     text += tr( "Contact: " ) + QString( "<a href=\"mailto:%1\">%2</a>" ).arg( mailTo ).arg( mailTo );
     text += "</p><p>";
     text += QString( "<p><a href=\"%1\">%2</a></p>" ).arg( homePage ).arg( homePage );
-    text += tr( "License: GPL V2 (GNU Public License Version 2)" );
+    text += tr( "License: GPL V2 (GNU General Public License Version 2)" );
     text += "</p><p>";
     text += tr( "This is free Open Source software, provided to you hoping that it might be "
 		"useful for you. It does not cost you anything, but on the other hand there "
@@ -860,6 +1120,25 @@ void MainWindow::showAboutDialog()
 }
 
 
+void MainWindow::showDonateDialog()
+{
+    QString dUrl = "https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=EYJXAVLGNRR5W";
+
+    QString text = "<h2>Donate</h2>";
+    text += "<p>";
+    text += tr( "QDirStat is Free Open Source Software. "
+                "You are not required to pay anything. "
+                "Donations are most welcome, of course." );
+    text += "</p><p>";
+    text += tr( "You can donate any amount of your choice:" );
+    text += "</p><p>";
+    text += QString( "<a href=\"%1\">QDirStat at PayPal</a>" ).arg(_dUrl );
+    text += "</p><p>";
+    text += tr( "(external browser window)" );
+    text += "</p>";
+
+    QMessageBox::about( this, tr( "Donate" ), text );
+}
 
 
 
@@ -898,6 +1177,7 @@ void MainWindow::itemClicked( const QModelIndex & index )
 void MainWindow::selectionChanged()
 {
     showSummary();
+    updateFileDetailsView();
 
     if ( _verboseSelection )
     {
@@ -911,6 +1191,9 @@ void MainWindow::currentItemChanged( FileInfo * newCurrent, FileInfo * oldCurren
 {
     showSummary();
 
+    if ( ! oldCurrent )
+	updateFileDetailsView();
+
     if ( _verboseSelection )
     {
 	logDebug() << "new current: " << newCurrent << endl;
@@ -920,7 +1203,7 @@ void MainWindow::currentItemChanged( FileInfo * newCurrent, FileInfo * oldCurren
 }
 
 
-QString MainWindow::formatTime( qint64 millisec )
+QString MainWindow::formatTime( qint64 millisec, bool showMillisec )
 {
     QString formattedTime;
     int hours;
@@ -938,7 +1221,12 @@ QString MainWindow::formatTime( qint64 millisec )
 
     if ( hours < 1 && min < 1 && sec < 60 )
     {
-	formattedTime.sprintf ( "%2d.%03lld ", sec, millisec );
+	if ( showMillisec )
+	    formattedTime.sprintf ( "%2d.%03lld ", sec, millisec );
+	else
+	    formattedTime.sprintf ( "%2d ", sec );
+	    // formattedTime.sprintf ( "%2d.%1lld ", sec, millisec / 100 );
+
 	formattedTime += tr( "sec" );
     }
     else
