@@ -15,8 +15,6 @@
 #include "MimeCategoryConfigPage.h"
 #include "DirTree.h"
 #include "DirInfo.h"
-#include "FileInfo.h"
-#include "FileInfo.h"
 #include "Logger.h"
 #include "Exception.h"
 
@@ -58,7 +56,7 @@ MimeCategoryConfigPage::MimeCategoryConfigPage( QWidget * parent ):
 
 MimeCategoryConfigPage::~MimeCategoryConfigPage()
 {
-    logDebug() << "MimeCategoryConfigPage destructor" << endl;
+    // logDebug() << "MimeCategoryConfigPage destructor" << endl;
 
     _ui->treemapView->setDirTree(0);
 
@@ -76,7 +74,7 @@ void MimeCategoryConfigPage::setup()
 
 void MimeCategoryConfigPage::applyChanges()
 {
-    logDebug() << endl;
+    // logDebug() << endl;
 
     save( value( listWidget()->currentItem() ) );
     _categorizer->writeSettings();
@@ -85,7 +83,7 @@ void MimeCategoryConfigPage::applyChanges()
 
 void MimeCategoryConfigPage::discardChanges()
 {
-    logDebug() << endl;
+    // logDebug() << endl;
 
     listWidget()->clear();
     _categorizer->clear();
