@@ -9,6 +9,10 @@
 #ifndef Version_h
 #define Version_h
 
-#define QDIRSTAT_VERSION "1.4"
+#define QDIRSTAT_VERSION "1.5"
+
+#define RELEASE_VERSION  "1.5"
+#define RELEASE_BASE_URL "https://github.com/shundhammer/qdirstat/releases/tag/"
+#define RELEASE_URL      RELEASE_BASE_URL RELEASE_VERSION
 
 #endif // Version_h
